/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.richfaces.realworld.manager;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.AutoCreate;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.contexts.Contexts;
import org.jboss.seam.core.Events;
import org.jboss.seam.faces.FacesMessages;
import org.richfaces.realworld.domain.Album;
import org.richfaces.realworld.domain.Image;
import org.richfaces.realworld.domain.Shelf;
import org.richfaces.realworld.domain.User;
import org.richfaces.realworld.service.Constants;
import org.richfaces.realworld.service.FlushStrategy;
import org.richfaces.realworld.service.IAlbumAction;

@Name("albumManager")
@Scope(ScopeType.EVENT)
@AutoCreate
public class AlbumManager implements Serializable{

	private static final long serialVersionUID = 2631634926126857691L;
	private boolean validationSuccess = false;
	@In private IAlbumAction albumAction;
	
	@In private User user;
	
	@In FileManager fileManager;
	@In FacesMessages facesMessages;
	@In @Out Model model;
	
	public void addAlbum(Album album){
		if(album.getShelf() == null){
			facesMessages.addToControl("shelf", "Shelf must be not-null", null);
			Contexts.getConversationContext().set("album", album);
			return;
		}
		validationSuccess = true;
		albumAction.addAlbum(album);
		fileManager.addDirectory(user.getLogin(), album.getId().toString());
		model.resetModel(NavigationEnum.ALBUM_PREVIEW, album.getShelf().getOwner(), album.getShelf(), album, null);
		model.setImages(new ArrayList<Image>());
		Contexts.getConversationContext().set("album", null);
		Events.instance().raiseEvent("albumAdded");
		Events.instance().raiseEvent("clearTree");
	}
	
	public void createAlbum(Shelf shelf){
		Album album = new Album();
		if(shelf == null){
			if(user.getShelfs().size() > 0){
				shelf = user.getShelfs().get(0);
			}
			if(shelf == null){
				Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, new Exception("You must create at least one shelf before creating album!"));
				return;
			}
		}
		album.setShelf(shelf);
		Contexts.getConversationContext().set("album", album);
	}

	public void editAlbum(Album album){
		albumAction.editAlbum(album);
		model.resetModel(NavigationEnum.ALBUM_PREVIEW, model.getSelectedUser(), model.getSelectedShelf(), album, null);
		Events.instance().raiseEvent("albumEdited");
	}
	
	public void deleteAlbum(Album album){
		//Update domain model
		String string = album.getId().toString();
		albumAction.deleteAlbum(album);
		//TODO nick - should be deleteDirectory(album). And call file manager directly
		deleteDirectory(string);
		//TODO nick - why not call just model.setAlbum(null). More: is deleted album the ones currently selected in model?
		model.resetModel(NavigationEnum.ALL_ALBUMS, model.getSelectedUser(), model.getSelectedShelf(), null, null);
		Events.instance().raiseEvent("albumDeleted");
		Events.instance().raiseEvent("clearTree");
	}
	
	public List<Image> getUnvisitedImages(Album album){
		if(album.getCountUnvisitedImages() == 0){
			album.setUnvisitedImages(albumAction.getUnvisitedImages(album));
		}
		return album.getUnvisitedImages();
	}
	
	private void deleteDirectory(String albumName) {
		fileManager.deleteDirectory(user.getLogin(), albumName);
	}

	public boolean isValidationSuccess() {
		return validationSuccess;
	}

	public void setValidationSuccess(boolean validationSuccess) {
		this.validationSuccess = validationSuccess;
	}
}
