/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.richfaces.realworld.manager;

import java.io.Serializable;
import java.util.List;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.AutoCreate;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Observer;
import org.jboss.seam.annotations.Scope;
import org.richfaces.realworld.domain.Album;
import org.richfaces.realworld.domain.Image;
import org.richfaces.realworld.domain.User;
import org.richfaces.realworld.service.IUserAction;

@Name("userManager")
@Scope(ScopeType.CONVERSATION)
@AutoCreate
public class UserManager implements Serializable{
	
	private static final long serialVersionUID = 6027103521084558931L;
	
	@In private User user;
	
	private Long countImages;
	private Long countAlbums;
	@In private IUserAction userAction;

	public Long countImages(User u) {
		if(null == countImages ){
			countImages = userAction.countImages(u);
		}
		return countImages;
	}
	
	public Long countSharedImages(User u) {
		return userAction.countSharedImages(u);
	}

	public Long countSharedAlbums(User u) {
		return userAction.countSharedAlbums(user);
	}
	
	public Long countAlbums(User u) {
		if(null == countAlbums ){
			countAlbums = userAction.countAlbums(user);
		}
		return countAlbums;
	}
	
	public List<Album> getSharedAlbums(User user){
		return userAction.getSharedAlbums(user);
	}
	
	public List<Image> getSharedImages(User user){
		return userAction.getSharedImages(user);
	}
	
	@Observer(value = "imageAdded", create = false)
	public void onImageAdded() {
		updateStatistics(countAlbums, countImages + 1);
	}

	@Observer(value = "imageDeleted", create = false)
	public void onImageDeleted(){
		updateStatistics(countAlbums, countImages - 1);
	}
	
	@Observer(value = "albumDeleted", create = false)
	public void onAlbumDeleted(){
		updateStatistics(countAlbums-1, countImages);
	}
	
	@Observer(value = "albumAdded", create = false)
	public void onAlbumAdded(){
		updateStatistics(countAlbums+1, countImages);
	}

	@Observer(value = "fileAdded", create = false)
	public void onFileAdded(){
		updateStatistics(countAlbums, countImages+1);
	}
	
	private void updateStatistics(Long i, Long j) {
		countAlbums = i;
		countImages = j;
	}
	
}
